#ifndef __SWITCH_PANEL_H__
#define __SWITCH_PANEL_H__

#include "hsf.h"
#include <stdint.h>
#include <stdbool.h>
#include "gpio.h"
#include "pinctrl.h"
#include "timer.h"
#include "platform_core_rom.h"
#include "osal_task.h"
#include "hfflash.h"
#include "hfuart.h"

//====================== 硬件引脚定义 ======================
// 根据CLAUDE.md中的物理连接定义，使用标准GPIO枚举
// LED指示灯引脚 (控制按键指示灯颜色: 高电平=白灯, 低电平=黄灯)
#define LED1_GPIO           GPIO_00               // GPIO 0 - 按键1指示灯
#define LED2_GPIO           GPIO_02               // GPIO 2 - 按键2指示灯  
#define LED3_GPIO           GPIO_05               // GPIO 5 - 按键3指示灯
#define LED4_GPIO           GPIO_10               // GPIO 10 - 按键4指示灯

// 开关控制引脚 (控制实际开关: 高电平=开, 低电平=关)
#define SWITCH1_GPIO        GPIO_12               // GPIO 12 - 开关1控制
#define SWITCH2_GPIO        GPIO_13               // GPIO 13 - 开关2控制
#define SWITCH3_GPIO        GPIO_14               // GPIO 14 - 开关3控制
#define SWITCH4_GPIO        GPIO_07               // GPIO 7 - 开关4控制

// 物理按键输入引脚 (输入检测: 低电平=按下, 高电平=松开)  
#define KEY1_GPIO           GPIO_09               // GPIO 9 - 按键1输入
#define KEY2_GPIO           GPIO_04               // GPIO 4 - 按键2输入
#define KEY3_GPIO           GPIO_11               // GPIO 11 - 按键3输入
#define KEY4_GPIO           GPIO_08               // GPIO 8 - 按键4输入

// 面板背光引脚 (控制面板背光: 高电平=亮黄灯, 低电平=灭灯)
#define PANEL_LED_GPIO      GPIO_03               // GPIO 3 - 面板背光

//====================== 系统常量定义 ======================
#define SWITCH_COUNT        4                     // 开关数量
#define KEY_DEBOUNCE_MS     30                    // 按键防抖时间(毫秒)
#define KEY_LONG_PRESS_MS   10000                 // 长按时间阈值(毫秒) - 用于进入配网
#define CONFIG_TIMEOUT_MS   (10*60*1000)          // 配网超时时间(10分钟)
#define CONFIG_BLINK_MS     (3*60*1000)           // 配网前3分钟闪烁时间
#define PANEL_BLINK_MS      1000                  // 面板背光快闪时间(1秒)
#define LED_BLINK_FREQ_HZ   1                     // 配网时LED闪烁频率(1Hz)

//====================== 设备状态定义 ======================
// 开关状态
typedef enum {
    SWITCH_OFF = 0,
    SWITCH_ON = 1
} switch_state_t;

// 按键状态
typedef enum {
    KEY_RELEASED = 1,  // 高电平 - 按键松开
    KEY_PRESSED = 0    // 低电平 - 按键按下
} key_state_t;

// LED状态
typedef enum {
    LED_YELLOW = 0,    // 低电平 - 黄灯
    LED_WHITE = 1      // 高电平 - 白灯
} led_state_t;

// 面板背光状态
typedef enum {
    PANEL_LED_OFF = 0,   // 低电平 - 背光关闭
    PANEL_LED_ON = 1     // 高电平 - 背光开启(黄色)
} panel_led_state_t;

// 系统工作模式
typedef enum {
    MODE_NORMAL = 0,        // 正常工作模式
    MODE_CONFIG,            // 配网模式
    MODE_FACTORY_TEST,      // 产测模式
    MODE_UNBIND            // 设备解绑模式
} system_mode_t;

//====================== 设备状态结构 ======================
// 单个开关状态信息
typedef struct {
    bool switch_on;         // 开关状态 (true=开, false=关)
    bool led_state;         // LED状态 (true=白灯, false=黄灯)
    bool physical_key;      // 物理按键状态 (true=松开, false=按下)
} switch_info_t;


// 系统设备状态
typedef struct {
    switch_info_t switches[SWITCH_COUNT];   // 4个开关的状态
    bool master_switch;                     // 总开关状态
    bool panel_led;                         // 面板背光状态
    bool is_bound;                          // 设备绑定状态
    bool is_first_boot;                     // 是否第一次上电
    system_mode_t mode;                     // 系统工作模式
    uint32_t reserved[10];                  // 保留字段
} device_state_t;


//====================== Flash存储相关 ======================
// Flash存储地址定义
#define SUPPORT_SAVE_TO_FLASH 1

// 定义Flash存储地址和大小
#define DEVICE_DATA_FLASH_SIZE    0x001000  // 使用一个页的大小
#define DEVICE_DATA_FLASH_ADDR    0x000000  // 主数据区地址
#define DEVICE_DATA_BACKUP_ADDR   (DEVICE_DATA_FLASH_ADDR + DEVICE_DATA_FLASH_SIZE)  // 备份数据区地址

// 设备数据结构
typedef struct {
    uint8_t checksum;           // 校验和
    uint32_t magic;             // 魔数，用于验证数据有效性
    uint32_t version;           // 版本号
    uint32_t data_len;          // 数据长度
    uint8_t data[0];            // 数据
} device_data_t;

#define DEVICE_DATA_MAGIC       0x4C505426  // "LPT&"的ASCII码
#define DEVICE_DATA_VERSION     1         // 数据版本号

//====================== 配网相关定义 ======================
#define FACTORY_TEST_SSID   "ShuorongSelfTest"     // 产测热点名称
#define CONFIG_ENTRY_COUNT  3                      // 连续重启次数进入配网

//====================== 任务和定时器相关 ======================
#define TASK_STACK_SIZE     2048
#define TASK_PRIORITY_HIGH  OSAL_TASK_PRIORITY_HIGH
#define TASK_PRIORITY_NORM  OSAL_TASK_PRIORITY_MIDDLE
#define TASK_PRIORITY_LOW   OSAL_TASK_PRIORITY_LOW

//====================== 全局变量声明 ======================
extern device_state_t g_device_state;
extern timer_handle_t g_key_debounce_timer[SWITCH_COUNT];
extern timer_handle_t g_config_timeout_timer;
extern timer_handle_t g_config_blink_timer;
extern osal_task *g_key_scan_task_handle;
extern osal_task *g_config_task_handle;

//====================== 函数声明 ======================

// 主程序函数
int switch_panel_main(void);

// 硬件控制函数
int key_system_init(void);
int switch_panel_gpio_init(void);
void set_switch_output(int switch_id, bool state);
void set_led_output(int led_id, led_state_t state);
void set_panel_led(panel_led_state_t state);
bool get_key_input(int key_id);

// 设备状态管理函数
int load_device_state(void);
int save_device_state(void);
void reset_device_state(void);
void sync_hardware_state(void);
void fast_report_switch(int switch_id);
void fast_report_master_switch(void);
void set_device_mode(system_mode_t mode);

// 开关控制函数
void update_switch_state(int switch_id, bool state);
void update_master_switch(bool state);
void apply_master_switch_control(void);

// 按键处理函数
int key_scan_task(void *arg);
void handle_key_press(int key_id);
void handle_key_long_press(int key_id);

// 配网相关函数
void enter_config_mode(void);
void exit_config_mode(void);
int config_mode_task(void *arg);
void config_led_blink(void);
void panel_led_blink(void);

// 产测相关函数
void enter_factory_test_mode(void);
void factory_test_sequence(void);
bool check_factory_test_wifi(void);

// HiLink设备事件处理
void handle_device_online(void);
void handle_device_offline(void);
void handle_device_unbind(void);
void sync_cloud_state(void);

// HiLink服务处理函数
int handle_put_switch(const char* svc_id, const char* payload, unsigned int len);
int handle_get_switch(const char* svc_id, const char* in, unsigned int in_len, char** out, unsigned int* out_len);
int handle_put_switch1(const char* svc_id, const char* payload, unsigned int len);
int handle_get_switch1(const char* svc_id, const char* in, unsigned int in_len, char** out, unsigned int* out_len);
int handle_put_switch2(const char* svc_id, const char* payload, unsigned int len);
int handle_get_switch2(const char* svc_id, const char* in, unsigned int in_len, char** out, unsigned int* out_len);
int handle_put_switch3(const char* svc_id, const char* payload, unsigned int len);
int handle_get_switch3(const char* svc_id, const char* in, unsigned int in_len, char** out, unsigned int* out_len);
int handle_put_switch4(const char* svc_id, const char* payload, unsigned int len);
int handle_get_switch4(const char* svc_id, const char* in, unsigned int in_len, char** out, unsigned int* out_len);

// 调试和工具函数
void print_device_state(void);
const char* get_mode_string(system_mode_t mode);

// 定时器回调函数
void key_debounce_timer_callback(uintptr_t data);
void config_timeout_timer_callback(uintptr_t data);
void config_blink_timer_callback(uintptr_t data);

#endif // __SWITCH_PANEL_H__