#include "stdint.h"
#include "hsf.h"
#include "hfsys.h"
#include "hfgpio.h"
#include "switch_panel.h"
#include "hilink.h"
#include "hsf.h"
#include "hfconfig.h"
#include "pinctrl.h"
#include "timer.h"
#include "soc_osal.h"
#include "systick.h"

//====================== 配网相关变量 ======================
static bool g_config_led_blink_state = false;
static bool g_panel_led_blink_state = false;
static uint32_t g_config_blink_start_time = 0;
static int g_config_key_id = -1;  // 触发配网的按键

//====================== 产测相关变量 ======================
static bool g_factory_test_running = false;
static int g_factory_test_step = 0;
static uint32_t g_factory_test_start_time = 0;

//====================== 配网模式函数 ======================

// 进入配网模式
void enter_config_mode(void) {
    if (g_device_state.mode == MODE_CONFIG) {
        e_printf("[CONFIG] 已在配网模式中\r\n");
        return;
    }
    
    e_printf("[CONFIG] 进入配网模式\r\n");
    
    // 更新设备模式
    set_device_mode(MODE_CONFIG);
    
    // 记录配网开始时间
    g_config_blink_start_time = hfsys_get_time();
    
    // 面板背光快闪1秒表示进入配网模式
    panel_led_blink();
    
    // 创建配网任务
    if (!g_config_task_handle) {
        g_config_task_handle = osal_kthread_create((osal_kthread_handler)config_mode_task, 
                                                  NULL, 
                                                  "config_task", 
                                                  TASK_STACK_SIZE);
        if (!g_config_task_handle) {
            e_printf("[CONFIG] 创建配网任务失败\r\n");
            return;
        }
        
        // 设置任务优先级
        osal_kthread_set_priority(g_config_task_handle, TASK_PRIORITY_NORM);
    }
    
    // 创建配网超时定时器
    if (!g_config_timeout_timer) {
        int ret = uapi_timer_create(TIMER_INDEX_1, &g_config_timeout_timer);
        if (ret != ERRCODE_SUCC) {
            e_printf("[CONFIG] 创建配网超时定时器失败: %d\r\n", ret);
        }
    }
    
    if (g_config_timeout_timer) {
        uapi_timer_start(g_config_timeout_timer, 
                        CONFIG_TIMEOUT_MS * 1000, 
                        config_timeout_timer_callback, 
                        0);
    }
    
    // 创建闪烁定时器
    if (!g_config_blink_timer) {
        int ret = uapi_timer_create(TIMER_INDEX_2, &g_config_blink_timer);
        if (ret != ERRCODE_SUCC) {
            e_printf("[CONFIG] 创建配网闪烁定时器失败: %d\r\n", ret);
        }
    }
    
    if (g_config_blink_timer) {
        uint32_t blink_period = 1000000 / (LED_BLINK_FREQ_HZ * 2);  // 半周期(微秒)
        uapi_timer_start(g_config_blink_timer, 
                        blink_period, 
                        config_blink_timer_callback, 
                        0);
    }
    
    e_printf("[CONFIG] 配网模式已开启，超时时间: %d分钟\r\n", CONFIG_TIMEOUT_MS / 60000);
}

// 退出配网模式
void exit_config_mode(void) {
    if (g_device_state.mode != MODE_CONFIG) {
        return;
    }
    
    e_printf("[CONFIG] 退出配网模式\r\n");
    
    // 停止定时器
    if (g_config_timeout_timer) {
        uapi_timer_stop(g_config_timeout_timer);
    }
    
    if (g_config_blink_timer) {
        uapi_timer_stop(g_config_blink_timer);
    }
    
    // 终止配网任务
    if (g_config_task_handle) {
        osal_kthread_destroy(g_config_task_handle, 1);
        g_config_task_handle = NULL;
    }
    
    // 恢复正常LED状态
    for (int i = 0; i < SWITCH_COUNT; i++) {
        set_led_output(i, g_device_state.switches[i].led_state ? LED_WHITE : LED_YELLOW);
    }
    
    // 恢复面板背光状态
    set_panel_led(g_device_state.panel_led ? PANEL_LED_ON : PANEL_LED_OFF);
    
    // 更新设备模式
    set_device_mode(MODE_NORMAL);
    
    // 重置配网相关变量
    g_config_key_id = -1;
    g_config_led_blink_state = false;
    g_panel_led_blink_state = false;
    
    e_printf("[CONFIG] 已退出配网模式\r\n");
}

// 配网模式任务
int config_mode_task(void *arg) {
    (void)arg;
    
    e_printf("[CONFIG] 配网任务开始\r\n");
    
    while (g_device_state.mode == MODE_CONFIG && !osal_kthread_should_stop()) {
        // 检查是否有产测热点
        if (check_factory_test_wifi()) {
            e_printf("[CONFIG] 检测到产测热点，进入产测模式\r\n");
            exit_config_mode();
            enter_factory_test_mode();
            break;
        }
        
        // 每秒检查一次
        osal_msleep(1000);
    }
    
    e_printf("[CONFIG] 配网任务结束\r\n");
    return 0;
}

// 配网LED闪烁
void config_led_blink(void) {
    uint32_t current_time = hfsys_get_time();
    uint32_t elapsed_time = current_time - g_config_blink_start_time;
    
    // 前3分钟闪烁，后7分钟常亮
    if (elapsed_time < CONFIG_BLINK_MS) {
        // 只有触发配网的按键LED闪烁，其他LED保持常亮
        for (int i = 0; i < SWITCH_COUNT; i++) {
            if (i == g_config_key_id) {
                // 触发配网的按键LED闪烁
                g_config_led_blink_state = !g_config_led_blink_state;
                set_led_output(i, g_config_led_blink_state ? LED_WHITE : LED_YELLOW);
            } else {
                // 其他按键LED保持常亮
                set_led_output(i, LED_YELLOW);
            }
        }
    } else {
        // 超过3分钟后，所有LED保持常亮
        for (int i = 0; i < SWITCH_COUNT; i++) {
            set_led_output(i, LED_YELLOW);
        }
    }
}

// 面板背光快闪
void panel_led_blink(void) {
    // 快闪1秒
    for (int i = 0; i < 10; i++) {
        set_panel_led(PANEL_LED_ON);
        osal_msleep(50);
        set_panel_led(PANEL_LED_OFF);
        osal_msleep(50);
    }
    
    // 恢复原状态
    set_panel_led(g_device_state.panel_led ? PANEL_LED_ON : PANEL_LED_OFF);
}

//====================== 产测模式函数 ======================

// 进入产测模式
void enter_factory_test_mode(void) {
    e_printf("[FACTORY] 进入产测模式\r\n");
    
    // 更新设备模式
    set_device_mode(MODE_FACTORY_TEST);
    
    // 初始化产测变量
    g_factory_test_running = true;
    g_factory_test_step = 0;
    g_factory_test_start_time = hfsys_get_time();
    
    // 开始产测序列
    factory_test_sequence();
}

// 产测序列
void factory_test_sequence(void) {
    e_printf("[FACTORY] 开始产测序列\r\n");
    
    // 步骤1: 开始指示 - 交替闪烁白黄灯500ms
    for (int i = 0; i < 6; i++) {
        for (int j = 0; j < SWITCH_COUNT; j++) {
            set_led_output(j, (i % 2) ? LED_WHITE : LED_YELLOW);
        }
        osal_msleep(500);
    }
    
    // 步骤2-5: 单个开关测试 (开1->4, 每个1.5秒)
    for (int switch_id = 0; switch_id < SWITCH_COUNT; switch_id++) {
        e_printf("[FACTORY] 测试开关%d\r\n", switch_id + 1);
        
        // 所有开关关闭，所有LED黄灯
        for (int i = 0; i < SWITCH_COUNT; i++) {
            set_switch_output(i, false);
            set_led_output(i, LED_YELLOW);
        }
        
        // 当前测试的开关开启且LED白灯
        set_switch_output(switch_id, true);
        set_led_output(switch_id, LED_WHITE);
        
        // 保持1.5秒
        osal_msleep(1500);
    }
    
    // 步骤6: 全关 (1.5秒)
    e_printf("[FACTORY] 测试全关\r\n");
    for (int i = 0; i < SWITCH_COUNT; i++) {
        set_switch_output(i, false);
        set_led_output(i, LED_YELLOW);
    }
    osal_msleep(1500);
    
    // 步骤7: 全开 (1.5秒)
    e_printf("[FACTORY] 测试全开\r\n");
    for (int i = 0; i < SWITCH_COUNT; i++) {
        set_switch_output(i, true);
        set_led_output(i, LED_WHITE);
    }
    osal_msleep(1500);
    
    // 步骤8: 再次全关
    e_printf("[FACTORY] 结束测试 - 全关\r\n");
    for (int i = 0; i < SWITCH_COUNT; i++) {
        set_switch_output(i, false);
        set_led_output(i, LED_YELLOW);
    }
    
    // TODO: WiFi信号强度校验
    bool wifi_test_pass = check_factory_test_wifi();
    
    // 显示最终结果
    if (wifi_test_pass) {
        e_printf("[FACTORY] 产测全部通过\r\n");
        // 所有开关开启表示通过
        for (int i = 0; i < SWITCH_COUNT; i++) {
            set_switch_output(i, true);
            set_led_output(i, LED_WHITE);
        }
    } else {
        e_printf("[FACTORY] 产测失败\r\n");
        // 所有开关关闭表示失败
        for (int i = 0; i < SWITCH_COUNT; i++) {
            set_switch_output(i, false);
            set_led_output(i, LED_YELLOW);
        }
    }
    
    // 产测完成，返回正常模式
    g_factory_test_running = false;
    set_device_mode(MODE_NORMAL);
    
    // 恢复设备默认状态
    sync_hardware_state();
    
    e_printf("[FACTORY] 产测序列完成\r\n");
}

// 检查产测热点
bool check_factory_test_wifi(void) {
    // 简化实现：直接返回false，实际应该扫描WiFi热点
    // TODO: 实际实现需要调用WiFi扫描接口查找 "ShuorongSelfTest" 热点
    
    return false;  // 默认未找到产测热点
}

//====================== 定时器回调函数 ======================

// 配网超时定时器回调
void config_timeout_timer_callback(uintptr_t data) {
    (void)data;
    
    e_printf("[CONFIG] 配网超时，退出配网模式\r\n");
    
    // 超时退出配网模式
    exit_config_mode();
}

// 配网闪烁定时器回调
void config_blink_timer_callback(uintptr_t data) {
    (void)data;
    uint32_t blink_period = 1000000 / (LED_BLINK_FREQ_HZ * 2);  // 半周期(微秒)
    // 执行闪烁操作
    config_led_blink();
    if (g_config_blink_timer) {
    uapi_timer_start(g_config_blink_timer, 
                        blink_period, 
                        config_blink_timer_callback, 
                        0);
    }
}