#include "hilink.h"
#include "securec.h"
#include "hilink_sal_defines.h"
#include "driver/pinctrl.h"
#include "driver/gpio.h"
#include "platform_core_rom.h"
#include "switch_panel.h"

//====================== 全局变量 ======================
device_state_t g_device_state = {
    .is_first_boot = true,
    .is_bound = false,
    .master_switch = false,
    .panel_led = true,
    .mode = MODE_NORMAL,
    .switches = {
        {.switch_on = false, .led_state = false, .physical_key = true},
        {.switch_on = false, .led_state = false, .physical_key = true},
        {.switch_on = false, .led_state = false, .physical_key = true},
        {.switch_on = false, .led_state = false, .physical_key = true}
    }
};
timer_handle_t g_key_debounce_timer[SWITCH_COUNT] = {0};
timer_handle_t g_config_timeout_timer = 0;
timer_handle_t g_config_blink_timer = 0;
osal_task *g_key_scan_task_handle = NULL;
osal_task *g_config_task_handle = NULL;

static bool g_initialized = false;
static uint32_t g_config_start_time = 0;
static bool g_config_led_state = false;

//====================== 存储管理函数 ======================

// 计算数据校验码

// 计算校验和
static uint8_t calculate_checksum(const uint8_t* data, int len)
{
    uint8_t sum = 0;
    for (int i = 0; i < len; i++) {
        sum ^= data[i];
    }
    return sum;
}

// 从指定地址读取设备数据
static bool read_device_data_from_addr(uint32_t addr, uint8_t* data, uint32_t len)
{
    int total_size = sizeof(device_data_t) + len;
    uint8_t checksum = 0;
    device_data_t *data_all = malloc(total_size);
    if (data_all == NULL) {
        e_printf("内存分配失败\r\n");
        return false;
    }
    
    int ret = hfuflash_read(addr, (char*)data_all, total_size);
    if (ret != total_size) {
        e_printf("从地址0x%x读取设备数据失败，实际读取长度：%d\r\n", addr, ret);
        goto lab_err;
    }

    checksum = calculate_checksum(data_all->data, data_all->data_len);
    if (checksum != data_all->checksum) {
        e_printf("check sum error:%x\n", data_all->version);
        goto lab_err;
    }

    if (data_all->magic != DEVICE_DATA_MAGIC) {
        e_printf("magic error:%x\n", data_all->magic);
        goto lab_err;
    }
    
    if (data_all->version > DEVICE_DATA_VERSION) {
        e_printf("versoin missmatch:%x\n", data_all->version);
        goto lab_err;
    }
    memcpy_s(data, len, data_all->data, data_all->data_len);
    free(data_all);
    return true;

lab_err:
    free(data_all);
    return false;
}

// 写入数据到指定地址
static bool write_device_data_to_addr(uint32_t addr, uint8_t* data, uint32_t len)
{
    int ret = hfuflash_erase_page(addr, 1);
    if (ret != HF_SUCCESS) {
        e_printf("擦除地址0x%x的Flash页失败，错误码：%d\r\n", addr, ret);
        return false;
    }
    uint32_t total_size = sizeof(device_data_t) + len;
    device_data_t *data_all = malloc(total_size);
    if (data_all == NULL) {
        e_printf("内存分配失败\r\n");
        return false;
    }
    memset(data_all, 0, total_size);
    memcpy_s(data_all->data, len, data, len);
    data_all->checksum = calculate_checksum(data_all->data, data_all->data_len);
    data_all->magic = DEVICE_DATA_MAGIC;
    data_all->version = DEVICE_DATA_VERSION;
    ret = hfuflash_write(addr, (char*)data_all, total_size);
    if (ret != total_size) {
        e_printf("写入数据到地址0x%x失败，实际写入长度：%d\r\n", addr, ret);
        free(data_all);
        return false;
    }
    free(data_all);
    return true;
}


// 从 Flash 加载设备状态
int load_device_state(void) {
    int ret = 0;
    device_state_t state;
    bool valid = false;
    
    // 尝试读取主数据区
    valid = read_device_data_from_addr(DEVICE_DATA_FLASH_ADDR, (uint8_t*)&state, sizeof(state));
    
    // 如果主数据区无效，尝试读取备份区
    if (!valid) {
        valid = read_device_data_from_addr(DEVICE_DATA_BACKUP_ADDR, (uint8_t*)&state, sizeof(state));
    }
    
    // 两个存储区都失败，使用默认状态  
    if (!valid) {
        e_printf("[STORAGE] 存储区数据损坏，使用默认状态\r\n");
        reset_device_state();
        save_device_state();
        return 0;
    }
    // 更新设备控制状态
    e_printf("设备状态恢复:\r\n");
    e_printf("首次启动: %d => %d\r\n", g_device_state.is_first_boot, state.is_first_boot);
    e_printf("配网状态: %d => %d\r\n", g_device_state.is_bound, state.is_bound);
    e_printf("总开关: %d => %d\r\n", g_device_state.master_switch, state.master_switch);
    e_printf("面板背光: %d => %d\r\n", g_device_state.panel_led, state.panel_led);
    e_printf("工作模式: %d => %d\r\n", g_device_state.mode, state.mode);
    for (int i = 0; i < SWITCH_COUNT; i++) {
        e_printf("开关%d: %d => %d, LED%d: %d => %d, 物理按键%d: %d => %d\r\n", 
                i + 1, g_device_state.switches[i].switch_on, state.switches[i].switch_on,
                i + 1, g_device_state.switches[i].led_state, state.switches[i].led_state,
                i + 1, g_device_state.switches[i].physical_key, state.switches[i].physical_key);
    }
    
    memcpy(&g_device_state, &state, sizeof(device_state_t));

    return 0;
}

// 保存设备状态到 Flash
int save_device_state(void) {
    int ret = 0;
    static device_state_t state = {};
    bool valid = false;
    if (state.is_first_boot == g_device_state.is_first_boot &&
        state.is_bound == g_device_state.is_bound &&
        state.master_switch == g_device_state.master_switch &&
        state.panel_led == g_device_state.panel_led &&
        state.mode == g_device_state.mode)
    {
        for (int i = 0; i < SWITCH_COUNT; i++) {
            if (state.switches[i].switch_on != g_device_state.switches[i].switch_on
                || state.switches[i].led_state != g_device_state.switches[i].led_state
                || state.switches[i].physical_key != g_device_state.switches[i].physical_key) {
                valid = true;
                break;
            }
        }
        if (!valid) {
            e_printf("[STORAGE] 设备状态未变化，跳过保存\r\n");
            return HF_SUCCESS;
        }
    }   
    // 准备数据
    memcpy(&state, &g_device_state, sizeof(device_state_t));
    // 保存到主存储区
    ret = write_device_data_to_addr(DEVICE_DATA_FLASH_ADDR, (uint8_t*)&state, sizeof(state));
    if (ret != HF_SUCCESS) {
        e_printf("[STORAGE] 写入主存储区失败: %d\r\n", ret);
        return ret;
    }
    
    // 保存到备份区
    ret = write_device_data_to_addr(DEVICE_DATA_BACKUP_ADDR, (uint8_t*)&state, sizeof(state));
    if (ret != HF_SUCCESS) {
        e_printf("[STORAGE] 写入备份区失败: %d\r\n", ret);
    }
    e_printf("[STORAGE] 设备状态保存完成\r\n");
    return HF_SUCCESS;
}

// 重置设备状态为默认值
void reset_device_state(void) {
    // memset(&g_device_state, 0, sizeof(device_data_t));
    
    // 设置默认状态
    g_device_state.master_switch = false;     // 总开关关闭
    g_device_state.panel_led = true;          // 面板背光开启
    g_device_state.is_bound = false;          // 设备未绑定
    g_device_state.is_first_boot = false;     // 标记为非首次启动
    g_device_state.mode = MODE_NORMAL;        // 正常模式
    
    // 所有开关默认关闭，所有LED默认黄灯
    for (int i = 0; i < SWITCH_COUNT; i++) {
        g_device_state.switches[i].switch_on = false;   // 开关关闭
        g_device_state.switches[i].led_state = false;   // LED黄灯
        g_device_state.switches[i].physical_key = true; // 按键松开
    }
    
    e_printf("[STATE] 设备状态已重置为默认值\r\n");
}

//====================== GPIO配置数据结构 ======================
typedef struct {
    pin_t pin;
    gpio_direction_t direction;
    const char* name;
} gpio_config_t;

// GPIO初始化配置表
static const gpio_config_t gpio_configs[] = {
    // 开关控制 - 输出
    {SWITCH1_GPIO, GPIO_DIRECTION_OUTPUT, "SWITCH1"},
    {SWITCH2_GPIO, GPIO_DIRECTION_OUTPUT, "SWITCH2"},
    {SWITCH3_GPIO, GPIO_DIRECTION_OUTPUT, "SWITCH3"},
    {SWITCH4_GPIO, GPIO_DIRECTION_OUTPUT, "SWITCH4"},
    
    // 物理按键 - 输入
    {KEY1_GPIO, GPIO_DIRECTION_INPUT, "KEY1"},
    {KEY2_GPIO, GPIO_DIRECTION_INPUT, "KEY2"},
    {KEY3_GPIO, GPIO_DIRECTION_INPUT, "KEY3"},
    {KEY4_GPIO, GPIO_DIRECTION_INPUT, "KEY4"},
    
    // 面板背光 - 输出
    {PANEL_LED_GPIO, GPIO_DIRECTION_OUTPUT, "PANEL_LED"},

    // LED指示灯 - 输出
    {LED1_GPIO, GPIO_DIRECTION_OUTPUT, "LED1"},
    {LED2_GPIO, GPIO_DIRECTION_OUTPUT, "LED2"},
    {LED3_GPIO, GPIO_DIRECTION_OUTPUT, "LED3"},
    // {LED4_GPIO, GPIO_DIRECTION_OUTPUT, "LED4"},
    
};

#define GPIO_CONFIG_COUNT (sizeof(gpio_configs) / sizeof(gpio_config_t))

//====================== 硬件控制函数 ======================

// GPIO 初始化
int switch_panel_gpio_init(void) {
    int ret = HF_SUCCESS;
    
    // 初始化pinctrl和GPIO
    uapi_pin_init();
    // uapi_gpio_init();
    
    // 使用循环配置所有GPIO
    for (uint32_t i = 0; i < GPIO_CONFIG_COUNT; i++) {
        const gpio_config_t* config = &gpio_configs[i];
        
        // 设置为GPIO模式
        ret = uapi_pin_set_mode(config->pin, PIN_MODE_0);
        if (ret != HF_SUCCESS) {
            e_printf("[GPIO] %s pinctrl初始化失败: %d\r\n", config->name, ret);
            return ret;
        }
        
        // 设置GPIO方向
        ret = uapi_gpio_set_dir(config->pin, config->direction);
        if (ret != HF_SUCCESS) {
            e_printf("[GPIO] %s 设置方向失败: %d\r\n", config->name, ret);
            return ret;
        }
        
        e_printf("[GPIO] %s 初始化完成 (方向: %s)\r\n", 
                config->name, 
                config->direction == GPIO_DIRECTION_OUTPUT ? "输出" : "输入");
    }
    
    e_printf("[GPIO] 所有GPIO初始化完成 (共%d个)\r\n", GPIO_CONFIG_COUNT);
    return HF_SUCCESS;
}

// 设置开关输出状态
void set_switch_output(int switch_id, bool state) {
    if (switch_id < 0 || switch_id >= SWITCH_COUNT) {
        e_printf("[HW] 无效的开关ID: %d\r\n", switch_id);
        return;
    }
    
    pin_t gpio_pin;
    switch (switch_id) {
        case 0: gpio_pin = SWITCH1_GPIO; break;  // SWITCH1
        case 1: gpio_pin = SWITCH2_GPIO; break;  // SWITCH2
        case 2: gpio_pin = SWITCH3_GPIO; break;  // SWITCH3
        case 3: gpio_pin = SWITCH4_GPIO; break;  // SWITCH4
        default: return;
    }
    
    // 设置输出状态
    gpio_level_t level = state ? GPIO_LEVEL_HIGH : GPIO_LEVEL_LOW;
    uapi_gpio_set_val(gpio_pin, level);
    e_printf("[HW] 开关%d 状态: %s\r\n", switch_id + 1, state ? "开" : "关");
}

// 设置 LED 指示灯状态
void set_led_output(int led_id, led_state_t state) {
    if (led_id < 0 || led_id >= SWITCH_COUNT) {
        e_printf("[HW] 无效的LED ID: %d\r\n", led_id);
        return;
    }
    
    pin_t gpio_pin;
    switch (led_id) {
        case 0: gpio_pin = LED1_GPIO; break;  // LED1
        case 1: gpio_pin = LED2_GPIO; break;  // LED2
        case 2: gpio_pin = LED3_GPIO; break;  // LED3
        case 3: gpio_pin = LED4_GPIO; break;  // LED4
        default: return;
    }
    
    // 设置LED状态 (高电平=白灯, 低电平=黄灯)
    gpio_level_t level = (state == LED_WHITE) ? GPIO_LEVEL_HIGH : GPIO_LEVEL_LOW;
    uapi_gpio_set_val(gpio_pin, level);
}

// 设置面板背光状态
void set_panel_led(panel_led_state_t state) {
    // 设置面板背光状态 (高电平=亮黄灯, 低电平=灭灯)
    gpio_level_t level = (state == PANEL_LED_ON) ? GPIO_LEVEL_HIGH : GPIO_LEVEL_LOW;
    uapi_gpio_set_val(PANEL_LED_GPIO, level);
}

// 获取按键输入状态
bool get_key_input(int key_id) {
    if (key_id < 0 || key_id >= SWITCH_COUNT) {
        return true;  // 默认松开状态
    }
    
    pin_t gpio_pin;
    switch (key_id) {
        case 0: gpio_pin = KEY1_GPIO; break;  // KEY1
        case 1: gpio_pin = KEY2_GPIO; break;  // KEY2
        case 2: gpio_pin = KEY3_GPIO; break;  // KEY3
        case 3: gpio_pin = KEY4_GPIO; break;  // KEY4
        default: return true;
    }
    
    // 获取输入状态 (低电平=按下, 高电平=松开)
    gpio_level_t level = uapi_gpio_get_val(gpio_pin);
    return (level == GPIO_LEVEL_HIGH);
}

//====================== 设备状态同步函数 ======================

// 同步硬件状态与软件状态
void sync_hardware_state(void) {
    // 同步所有开关状态
    for (int i = 0; i < SWITCH_COUNT; i++) {
        set_switch_output(i, g_device_state.switches[i].switch_on);
        set_led_output(i, g_device_state.switches[i].led_state ? LED_WHITE : LED_YELLOW);
    }
    
    // 同步面板背光状态
    set_panel_led(g_device_state.panel_led ? PANEL_LED_ON : PANEL_LED_OFF);
    
    e_printf("[STATE] 硬件状态已同步\r\n");
}

//====================== 主程序函数 ======================

int switch_panel_main(void) {
    int ret = HF_SUCCESS;
    
    if (g_initialized) {
        e_printf("[MAIN] 开关面板已初始化\r\n");
        return HF_SUCCESS;
    }
    g_initialized = true;
    e_printf("[MAIN] 开始初始化SORONTEK智能面板...\r\n");
        
    // 加载设备状态
    ret = load_device_state();
    if (ret != HF_SUCCESS) {
        e_printf("[MAIN] 加载设备状态失败: %d\r\n", ret);
        return ret;
    }
    // 检查是否首次启动，如果是则标记为非首次
    bool first_boot = g_device_state.is_first_boot;
    if (g_device_state.is_first_boot) {
        g_device_state.is_first_boot = false;  // 标记为非首次启动
        e_printf("[MAIN] 检测到首次启动\r\n");
    }
    // 初始化GPIO
    ret = switch_panel_gpio_init();
    if (ret != HF_SUCCESS) {
        e_printf("[MAIN] GPIO初始化失败: %d\r\n", ret);
        return ret;
    }
    if(key_system_init() != HF_SUCCESS) {
        e_printf("[MAIN] 按键系统初始化失败: %d\r\n", ret);
        return ret;
    }
    
    // 同步硬件状态
    sync_hardware_state();
    
    // 保存状态更新
    save_device_state();
    
    // 配网逻辑：
    // 1. 如果设备未绑定 且 是第一次启动 -> 直接进入配网
    // 2. 如果设备未绑定 且 不是第一次启动 -> 等待按键触发配网
    if (!g_device_state.is_bound) {
        if (first_boot) {
            e_printf("[MAIN] 首次启动且未绑定，直接进入配网模式\r\n");
            enter_config_mode();
        } else {
            e_printf("[MAIN] 设备未绑定，等待按键触发配网\r\n");
            // 配网逻辑将在按键处理函数中实现
        }
    } else {
        e_printf("[MAIN] 设备已绑定，正常运行\r\n");
    }
    
    e_printf("[MAIN] SORONTEK智能面板初始化完成\r\n");
    print_device_state();
    
    return HF_SUCCESS;
}

//====================== 调试函数 ======================

void print_device_state(void) {
    e_printf("\r\n===== 设备状态信息 =====\r\n");
    e_printf("总开关: %s\r\n", g_device_state.master_switch ? "开" : "关");
    e_printf("面板背光: %s\r\n", g_device_state.panel_led ? "开" : "关");
    e_printf("绑定状态: %s\r\n", g_device_state.is_bound ? "已绑定" : "未绑定");
    e_printf("首次启动: %s\r\n", g_device_state.is_first_boot ? "是" : "否");
    e_printf("工作模式: %s\r\n", get_mode_string(g_device_state.mode));
    
    for (int i = 0; i < SWITCH_COUNT; i++) {
        e_printf("开关%d: %s, LED: %s\r\n", 
               i + 1,
               g_device_state.switches[i].switch_on ? "开" : "关",
               g_device_state.switches[i].led_state ? "白灯" : "黄灯");
    }
    e_printf("=======================\r\n\r\n");
}

const char* get_mode_string(system_mode_t mode) {
    switch (mode) {
        case MODE_NORMAL: return "正常模式";
        case MODE_CONFIG: return "配网模式";
        case MODE_FACTORY_TEST: return "产测模式";
        case MODE_UNBIND: return "解绑模式";
        default: return "未知模式";
    }
}